@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -22,9 +22,11 @@
 class CControllerPopupTriggerExpr extends CController {
 	private $metrics = [];
 	private $param1SecCount = [];
+	private $param1Period = [];
 	private $param1Sec = [];
 	private $param1Str = [];
 	private $param2SecCount = [];
+	private $param2SecMode = [];
 	private $param3SecVal = [];
 	private $param3SecPercent = [];
 	private $paramSecIntCount = [];
@@ -61,7 +63,21 @@ class CControllerPopupTriggerExpr extends CController {
 			],
 			'shift' => [
 				'C' => _('Time shift'),
-				'T' => T_ZBX_INT
+				'T' => T_ZBX_INT,
+				'A' => false
+			]
+		];
+
+		$this->param1Period = [
+			'last' => [
+				'C' => _('Last of').' (T)',
+				'T' => T_ZBX_INT,
+				'A' => true
+			],
+			'period_shift' => [
+				'C' => _('Period shift'),
+				'T' => T_ZBX_INT,
+				'A' => true
 			]
 		];
 
@@ -76,19 +92,35 @@ class CControllerPopupTriggerExpr extends CController {
 		$this->param1Str = [
 			'pattern' => [
 				'C' => 'T',
-				'T' => T_ZBX_STR
+				'T' => T_ZBX_STR,
+				'A' => false
 			]
 		];
 
 		$this->param2SecCount = [
 			'pattern' => [
 				'C' => 'V',
-				'T' => T_ZBX_STR
+				'T' => T_ZBX_STR,
+				'A' => false
 			],
 			'last' => [
 				'C' => _('Last of').' (T)',
 				'T' => T_ZBX_INT,
-				'M' => $this->metrics
+				'M' => $this->metrics,
+				'A' => false
+			]
+		];
+
+		$this->param2SecMode = [
+			'last' => [
+				'C' => _('Last of').' (T)',
+				'T' => T_ZBX_INT,
+				'A' => true
+			],
+			'mode' => [
+				'C' => 'Mode',
+				'T' => T_ZBX_STR,
+				'A' => false
 			]
 		];
 
@@ -101,15 +133,18 @@ class CControllerPopupTriggerExpr extends CController {
 			],
 			'v' => [
 				'C' => 'V',
-				'T' => T_ZBX_STR
+				'T' => T_ZBX_STR,
+				'A' => false
 			],
 			'o' => [
 				'C' => 'O',
-				'T' => T_ZBX_STR
+				'T' => T_ZBX_STR,
+				'A' => false
 			],
 			'shift' => [
 				'C' => _('Time shift'),
-				'T' => T_ZBX_INT
+				'T' => T_ZBX_INT,
+				'A' => false
 			]
 		];
 
@@ -122,7 +157,8 @@ class CControllerPopupTriggerExpr extends CController {
 			],
 			'shift' => [
 				'C' => _('Time shift'),
-				'T' => T_ZBX_INT
+				'T' => T_ZBX_INT,
+				'A' => false
 			],
 			'p' => [
 				'C' => _('Percentage').' (P)',
@@ -145,7 +181,8 @@ class CControllerPopupTriggerExpr extends CController {
 			],
 			'shift' => [
 				'C' => _('Time shift'),
-				'T' => T_ZBX_INT
+				'T' => T_ZBX_INT,
+				'A' => false
 			]
 		];
 
@@ -158,7 +195,8 @@ class CControllerPopupTriggerExpr extends CController {
 			],
 			'shift' => [
 				'C' => _('Time shift'),
-				'T' => T_ZBX_INT
+				'T' => T_ZBX_INT,
+				'A' => false
 			],
 			'time' => [
 				'C' => _('Time').' (t)',
@@ -167,11 +205,13 @@ class CControllerPopupTriggerExpr extends CController {
 			],
 			'fit' => [
 				'C' => _('Fit'),
-				'T' => T_ZBX_STR
+				'T' => T_ZBX_STR,
+				'A' => false
 			],
 			'mode' => [
 				'C' => _('Mode'),
-				'T' => T_ZBX_STR
+				'T' => T_ZBX_STR,
+				'A' => false
 			]
 		];
 
@@ -184,7 +224,8 @@ class CControllerPopupTriggerExpr extends CController {
 			],
 			'shift' => [
 				'C' => _('Time shift'),
-				'T' => T_ZBX_INT
+				'T' => T_ZBX_INT,
+				'A' => false
 			],
 			't' => [
 				'C' => _('Threshold'),
@@ -193,7 +234,8 @@ class CControllerPopupTriggerExpr extends CController {
 			],
 			'fit' => [
 				'C' => _('Fit'),
-				'T' => T_ZBX_STR
+				'T' => T_ZBX_STR,
+				'A' => false
 			]
 		];
 
@@ -321,7 +363,7 @@ class CControllerPopupTriggerExpr extends CController {
 				'operators' => ['=', '<>', '>', '<', '>=', '<=']
 			],
 			'fuzzytime' => [
-				'description' => _('fuzzytime() - Difference between item timestamp value and Zabbix server timestamp is over T seconds (1 - true, 0 - false)'),
+				'description' => _('fuzzytime() - Difference between item value (as timestamp) and Zabbix server timestamp is less than or equal to T seconds (1 - true, 0 - false)'),
 				'params' => $this->param1Sec,
 				'allowed_types' => $this->allowedTypesNumeric,
 				'operators' => ['=', '<>']
@@ -366,8 +408,8 @@ class CControllerPopupTriggerExpr extends CController {
 				'operators' => ['=', '<>', '>', '<', '>=', '<=']
 			],
 			'nodata' => [
-				'description' => _('nodata() - No data received during period of time T (1 - true, 0 - false)'),
-				'params' => $this->param1Sec,
+				'description' => _('nodata() - No data received during period of time T (1 - true, 0 - false), Mode (strict - ignore proxy time delay in sending data)'),
+				'params' => $this->param2SecMode,
 				'allowed_types' => $this->allowedTypesAny,
 				'operators' => ['=', '<>']
 			],
@@ -388,6 +430,42 @@ class CControllerPopupTriggerExpr extends CController {
 				'params' => $this->paramTimeleft,
 				'allowed_types' => $this->allowedTypesNumeric,
 				'operators' => ['=', '<>', '>', '<', '>=', '<=']
+			],
+			'trendavg' => [
+				'description' => _('trendavg() - Average value of a period T with exact period shift'),
+				'params' => $this->param1Period,
+				'allowed_types' => $this->allowedTypesNumeric,
+				'operators' => ['=', '<>', '>', '<', '>=', '<=']
+			],
+			'trendcount' => [
+				'description' => _('trendcount() - Number of successfully retrieved values V (which fulfill operator O) for period T with exact period shift'),
+				'params' => $this->param1Period,
+				'allowed_types' => $this->allowedTypesAny,
+				'operators' => ['=', '<>', '>', '<', '>=', '<=']
+			],
+			'trenddelta' => [
+				'description' => _('trenddelta() - Difference between MAX and MIN value of a period T with exact period shift'),
+				'params' => $this->param1Period,
+				'allowed_types' => $this->allowedTypesNumeric,
+				'operators' => ['=', '<>', '>', '<', '>=', '<=']
+			],
+			'trendmax' => [
+				'description' => _('trendmax() - Maximum value for period T with exact period shift'),
+				'params' => $this->param1Period,
+				'allowed_types' => $this->allowedTypesNumeric,
+				'operators' => ['=', '<>', '>', '<', '>=', '<=']
+			],
+			'trendmin' => [
+				'description' => _('trendmin() - Minimum value for period T with exact period shift'),
+				'params' => $this->param1Period,
+				'allowed_types' => $this->allowedTypesNumeric,
+				'operators' => ['=', '<>', '>', '<', '>=', '<=']
+			],
+			'trendsum' => [
+				'description' => _('trendsum() - Sum of values of a period T with exact period shift'),
+				'params' => $this->param1Period,
+				'allowed_types' => $this->allowedTypesNumeric,
+				'operators' => ['=', '<>', '>', '<', '>=', '<=']
 			]
 		];
 
@@ -406,7 +484,7 @@ class CControllerPopupTriggerExpr extends CController {
 			'dstfld1' =>			'string|not_empty',
 			'expression' =>			'string',
 			'itemid' =>				'db items.itemid',
-			'parent_discoveryid' =>	'int32',
+			'parent_discoveryid' =>	'db items.itemid',
 			'function' =>			'in '.implode(',', array_keys($this->functions)),
 			'operator' =>			'in '.implode(',', array_keys($this->operators)),
 			'params' =>				'',
@@ -427,7 +505,7 @@ class CControllerPopupTriggerExpr extends CController {
 
 			if ($this->hasInput('add')) {
 				$this->setResponse(
-					(new CControllerResponseData(['main_block' => CJs::encodeJson($output)]))->disableView()
+					(new CControllerResponseData(['main_block' => json_encode($output)]))->disableView()
 				);
 			}
 			else {
@@ -460,9 +538,11 @@ class CControllerPopupTriggerExpr extends CController {
 			$result = $expression_data->parse($expression);
 
 			if ($result) {
-				// Only one item function macro is supported in an expression.
-				$function_macro_tokens = $result->getTokensByType(CTriggerExpressionParserResult::TOKEN_TYPE_FUNCTION_MACRO);
-				if (count($function_macro_tokens) == 1) {
+				$function_macro_tokens = $result->getTokensByType(
+					CTriggerExprParserResult::TOKEN_TYPE_FUNCTION_MACRO
+				);
+
+				if ($function_macro_tokens) {
 					$function_macro_token = $function_macro_tokens[0];
 					$function = $function_macro_token['data']['functionName'];
 
@@ -480,20 +560,31 @@ class CControllerPopupTriggerExpr extends CController {
 					}
 
 					/*
-					 * Try to find an operator and a numeric value.
-					 * The value and operator can be extracted only if the immediately follow the item function macro.
+					 * Try to find an operator and a value.
+					 * The value and operator can be extracted only if they immediately follow the item function macro.
 					 */
 					$tokens = $result->getTokens();
 					foreach ($tokens as $key => $token) {
-						if ($token['type'] == CTriggerExpressionParserResult::TOKEN_TYPE_FUNCTION_MACRO) {
+						if ($token['type'] == CTriggerExprParserResult::TOKEN_TYPE_FUNCTION_MACRO) {
 							if (array_key_exists($key + 2, $tokens)
-									&& $tokens[$key + 1]['type'] == CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR
+									&& $tokens[$key + 1]['type'] == CTriggerExprParserResult::TOKEN_TYPE_OPERATOR
 									&& array_key_exists($function, $this->functions)
-									&& in_array($tokens[$key + 1]['value'], $this->functions[$function]['operators'])
-									&& $tokens[$key + 2]['type'] == CTriggerExpressionParserResult::TOKEN_TYPE_NUMBER) {
-
+									&& in_array($tokens[$key + 1]['value'],
+										$this->functions[$function]['operators'])) {
 								$operator = $tokens[$key + 1]['value'];
-								$value = $tokens[$key + 2]['value'];
+
+								$value = '';
+								$i = 2;
+
+								if (array_key_exists($key + 3, $tokens)
+										&& $tokens[$key + 2]['type'] == CTriggerExprParserResult::TOKEN_TYPE_OPERATOR) {
+									$value .= $tokens[$key + 2]['value'];
+									$i++;
+								}
+
+								$value .= ($tokens[$key + $i]['type'] == CTriggerExprParserResult::TOKEN_TYPE_STRING)
+									? $tokens[$key + $i]['data']['string']
+									: $tokens[$key + $i]['value'];
 							}
 							else {
 								break;
@@ -569,7 +660,7 @@ class CControllerPopupTriggerExpr extends CController {
 			'value' => $value,
 			'params' => $params,
 			'paramtype' => $param_type,
-			'description' => $description,
+			'item_description' => $description,
 			'functions' => $this->functions,
 			'function' => $function,
 			'operator' => $operator,
@@ -577,7 +668,7 @@ class CControllerPopupTriggerExpr extends CController {
 			'itemValueType' => $item_value_type,
 			'selectedFunction' => null,
 			'groupid' => $this->getInput('groupid', 0),
-			'hostid' => $this->getInput('hostid', 0),
+			'hostid' => $this->getInput('hostid', 0)
 		];
 
 		// Check if submitted function is usable with selected item.
@@ -610,11 +701,13 @@ class CControllerPopupTriggerExpr extends CController {
 		// Create and validate trigger expression before inserting it into textarea field.
 		if ($this->getInput('add', false)) {
 			try {
-				if ($data['description']) {
+				if ($data['item_description']) {
 					if ($data['paramtype'] == PARAM_TYPE_COUNTS
 							&& array_key_exists('last', $data['params'])
 							&& $data['params']['last'] !== '') {
-						$data['params']['last'] = '#'.$data['params']['last'];
+						$data['params']['last'] = zbx_is_int($data['params']['last'])
+							? '#'.$data['params']['last']
+							: $data['params']['last'];
 					}
 					elseif ($data['paramtype'] == PARAM_TYPE_TIME && in_array($function, ['last', 'band', 'strlen'])) {
 						$data['params']['last'] = '';
@@ -632,7 +725,7 @@ class CControllerPopupTriggerExpr extends CController {
 						$function,
 						rtrim(implode(',', $quoted_params), ','),
 						$operator,
-						$data['value']
+						CTriggerExpression::quoteString($data['value'])
 					);
 
 					// Validate trigger expression.
@@ -688,7 +781,7 @@ class CControllerPopupTriggerExpr extends CController {
 			}
 
 			$this->setResponse(
-				(new CControllerResponseData(['main_block' => CJs::encodeJson($output)]))->disableView()
+				(new CControllerResponseData(['main_block' => json_encode($output)]))->disableView()
 			);
 		}
 		else {
