@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ class CImportReferencer {
 	protected $iconMaps = [];
 	protected $maps = [];
 	protected $screens = [];
-	protected $templateScreens = [];
+	protected $templateDashboards = [];
 	protected $macros = [];
 	protected $proxies = [];
 	protected $hostPrototypes = [];
@@ -57,7 +57,7 @@ class CImportReferencer {
 	protected $iconMapsRefs;
 	protected $mapsRefs;
 	protected $screensRefs;
-	protected $templateScreensRefs;
+	protected $templateDashboardsRefs;
 	protected $macrosRefs;
 	protected $proxiesRefs;
 	protected $hostPrototypesRefs;
@@ -104,18 +104,33 @@ class CImportReferencer {
 	}
 
 	/**
-	 * Get template id by host.
+	 * Get interface ID by host ID and interface reference name.
+	 *
+	 * @param string $hostid  Host ID.
+	 * @param string $name    Interface reference name.
+	 *
+	 * @return string|bool
+	 */
+	public function resolveInterface($hostid, $name) {
+		return (array_key_exists($hostid, $this->interfacesCache)
+				&& array_key_exists($name, $this->interfacesCache[$hostid]))
+			? $this->interfacesCache[$hostid][$name]
+			: false;
+	}
+
+	/**
+	 * Get template id by template name.
 	 *
 	 * @param string $host
 	 *
 	 * @return string|bool
 	 */
-	public function resolveTemplate($host) {
+	public function resolveTemplate($name) {
 		if ($this->templatesRefs === null) {
 			$this->selectTemplates();
 		}
 
-		return isset($this->templatesRefs[$host]) ? $this->templatesRefs[$host] : false;
+		return isset($this->templatesRefs[$name]) ? $this->templatesRefs[$name] : false;
 	}
 
 	/**
@@ -274,20 +289,20 @@ class CImportReferencer {
 	}
 
 	/**
-	 * Get templated screen ID by template ID and screen name.
+	 * Get template dashboard ID by template ID and dashboard name.
 	 *
-	 * @param string $templateId
-	 * @param string $screenName
+	 * @param string $templateid
+	 * @param string $name
 	 *
 	 * @return string|bool
 	 */
-	public function resolveTemplateScreen($templateId, $screenName) {
-		if ($this->templateScreensRefs === null) {
-			$this->selectTemplateScreens();
+	public function resolveTemplateDashboards($templateid, $name) {
+		if ($this->templateDashboardsRefs === null) {
+			$this->selectTemplateDashboards();
 		}
 
-		return isset($this->templateScreensRefs[$templateId][$screenName])
-			? $this->templateScreensRefs[$templateId][$screenName]
+		return isset($this->templateDashboardsRefs[$templateid][$name])
+			? $this->templateDashboardsRefs[$templateid][$name]
 			: false;
 	}
 
@@ -608,28 +623,18 @@ class CImportReferencer {
 	 *
 	 * @param array $screens
 	 */
-	public function addTemplateScreens(array $screens) {
-		$this->templateScreens = array_unique(array_merge($this->templateScreens, $screens));
+	public function addTemplateDashboards(array $dashboards) {
+		$this->templateDashboards = array_unique(array_merge($this->templateDashboards, $dashboards));
 	}
 
 	/**
-	 * Add screen name association with screen id.
+	 * Add template dashboard name association with template dashboard ID.
 	 *
 	 * @param string $name
-	 * @param string $screenId
+	 * @param string $template_dashboardid
 	 */
-	public function addScreenRef($name, $screenId) {
-		$this->screensRefs[$name] = $screenId;
-	}
-
-	/**
-	 * Add template screen name association with template screen ID.
-	 *
-	 * @param string $screenName
-	 * @param string $templateScreenId
-	 */
-	public function addTemplateScreenRef($screenName, $templateScreenId) {
-		$this->templateScreensRefs[$screenName] = $templateScreenId;
+	public function addTemplateDashboardsRef($name, $template_dashboardid) {
+		$this->templateDashboardsRefs[$name] = $template_dashboardid;
 	}
 
 	/**
@@ -872,7 +877,7 @@ class CImportReferencer {
 			$this->valueMapsRefs = [];
 
 			$valuemaps = API::ValueMap()->get([
-				'output' => ['valeumapid', 'name'],
+				'output' => ['valuemapid', 'name'],
 				'filter' => ['name' => $this->valueMaps]
 			]);
 
@@ -1025,21 +1030,21 @@ class CImportReferencer {
 	}
 
 	/**
-	 * Select template screen IDs for previously added screen names and template IDs.
+	 * Select template dashboard IDs for previously added dashboard names and template IDs.
 	 */
-	protected function selectTemplateScreens() {
-		if ($this->templateScreens) {
-			$this->templateScreensRefs = [];
+	protected function selectTemplateDashboards() {
+		if ($this->templateDashboards) {
+			$this->templateDashboardsRefs = [];
 
-			$db_template_screens = API::TemplateScreen()->get([
-				'filter' => ['name' => $this->templateScreens],
-				'output' => ['screenid', 'name', 'templateid']
+			$db_template_dashboards = API::TemplateDashboard()->get([
+				'output' => ['dashboardid', 'name', 'templateid'],
+				'filter' => ['name' => $this->templateDashboards]
 			]);
-			foreach ($db_template_screens as $screen) {
-				$this->templateScreensRefs[$screen['templateid']][$screen['name']] = $screen['screenid'];
+			foreach ($db_template_dashboards as $dashboard) {
+				$this->templateDashboardsRefs[$dashboard['templateid']][$dashboard['name']] = $dashboard['dashboardid'];
 			}
 
-			$this->templateScreens = [];
+			$this->templateDashboards = [];
 		}
 	}
 
@@ -1111,7 +1116,7 @@ class CImportReferencer {
 					' FROM hosts h,host_discovery hd,items i'.
 					' WHERE h.hostid=hd.hostid'.
 						' AND hd.parent_itemid=i.itemid'.
-						' AND '.implode(' OR ', $sqlWhere)
+						' AND ('.implode(' OR ', $sqlWhere).')'
 				);
 				while ($data = DBfetch($query)) {
 					$this->hostPrototypesRefs[$data['parent_hostid']][$data['parent_itemid']][$data['host']] = $data['hostid'];
@@ -1186,7 +1191,7 @@ class CImportReferencer {
 					'SELECT ht.hostid,hs.httptestid,hs.name,hs.httpstepid'.
 					' FROM httptest ht,httpstep hs'.
 					' WHERE ht.httptestid=hs.httptestid'.
-						' AND '.implode(' OR ', $sql_where)
+						' AND ('.implode(' OR ', $sql_where).')'
 				);
 				while ($db_httpstep = DBfetch($db_httpsteps)) {
 					$this->httpstepsRefs[$db_httpstep['hostid']][$db_httpstep['httptestid']][$db_httpstep['name']] =
