@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -175,7 +175,7 @@ function make_event_details(array $event, array $allowed) {
 	$table = (new CTableInfo())
 		->addRow([
 			_('Event'),
-			$event['name']
+			(new CCol($event['name']))->addClass(ZBX_STYLE_WORDWRAP)
 		])
 		->addRow([
 			_('Operational data'),
@@ -418,264 +418,6 @@ function make_small_eventlist(array $startEvent, array $allowed) {
 	return $table;
 }
 
-/**
- * Create table with trigger description and events.
- *
- * @param array  $trigger                     An array of trigger data.
- * @param string $trigger['triggerid']        Trigger ID to select events.
- * @param string $trigger['comments']         Trigger description.
- * @param string $trigger['url']              Trigger URL.
- * @param string $eventid_till
- * @param array  $allowed                     An array of user role rules.
- * @param bool   $allowed['ui_problems']      Whether user is allowed to visit problems page.
- * @param bool   $allowed['add_comments']     Whether user is allowed to add problems comments.
- * @param bool   $allowed['change_severity']  Whether user is allowed to change problems severity.
- * @param bool   $allowed['acknowledge']      Whether user is allowed to acknowledge problems.
- * @param bool   $allowed['close']            Whether user is allowed to close problems.
- * @param bool   $show_timeline               Show time line flag.
- * @param int    $show_tags                   Show tags flag. Possible values:
- *                                              - PROBLEMS_SHOW_TAGS_NONE;
- *                                              - PROBLEMS_SHOW_TAGS_1;
- *                                              - PROBLEMS_SHOW_TAGS_2;
- *                                              - PROBLEMS_SHOW_TAGS_3 (default).
- * @param array  $filter_tags                 An array of tag filtering data.
- * @param string $filter_tags[]['tag']        Tag name.
- * @param int    $filter_tags[]['operator']   Tag operator.
- * @param string $filter_tags[]['value']      Tag value.
- * @param int    $tag_name_format             Tag name format. Possible values:
- *                                              - PROBLEMS_TAG_NAME_FULL (default);
- *                                              - PROBLEMS_TAG_NAME_SHORTENED;
- *                                              - PROBLEMS_TAG_NAME_NONE.
- * @param string $tag_priority                A list of comma-separated tag names.
- *
- * @return CDiv
- */
-function make_popup_eventlist($trigger, $eventid_till, $allowed, $show_timeline = true, $show_tags = PROBLEMS_SHOW_TAGS_3,
-		array $filter_tags = [], $tag_name_format = PROBLEMS_TAG_NAME_FULL, $tag_priority = '') {
-	// Show trigger description and URL.
-	$div = new CDiv();
-
-	if ($trigger['comments'] !== '') {
-		$div->addItem(
-			(new CDiv())
-				->addItem(zbx_str2links($trigger['comments']))
-				->addClass(ZBX_STYLE_OVERLAY_DESCR)
-				->addStyle('max-width: 500px')
-		);
-	}
-
-	if ($trigger['url'] !== '') {
-		$trigger_url = CHtmlUrlValidator::validate($trigger['url'], ['allow_user_macro' => false])
-			? $trigger['url']
-			: 'javascript: alert(\''._s('Provided URL "%1$s" is invalid.', zbx_jsvalue($trigger['url'], false, false)).
-				'\');';
-
-		$div->addItem(
-			(new CDiv())
-				->addItem(new CLink(CHTML::encode($trigger['url']), $trigger_url))
-				->addClass(ZBX_STYLE_OVERLAY_DESCR_URL)
-				->addStyle('max-width: 500px')
-		);
-	}
-
-	// indicator of sort field
-	$sort_div = (new CSpan())->addClass(ZBX_STYLE_ARROW_DOWN);
-
-	if ($show_timeline) {
-		$header = [
-			(new CColHeader([_('Time'), $sort_div]))->addClass(ZBX_STYLE_RIGHT),
-			(new CColHeader())->addClass(ZBX_STYLE_TIMELINE_TH),
-			(new CColHeader())->addClass(ZBX_STYLE_TIMELINE_TH)
-		];
-	}
-	else {
-		$header = [[_('Time'), $sort_div]];
-	}
-
-	// Select and show events.
-	$table = (new CTableInfo())
-		->setHeader(array_merge($header, [
-			_('Recovery time'),
-			_('Status'),
-			_('Duration'),
-			_('Ack'),
-			($show_tags != PROBLEMS_SHOW_TAGS_NONE) ? _('Tags') : null
-		]));
-
-	if ($eventid_till != 0) {
-		$problems = API::Event()->get([
-			'output' => ['eventid', 'r_eventid', 'clock', 'ns', 'objectid', 'acknowledged'],
-			'selectTags' => ['tag', 'value'],
-			'select_acknowledges' => ['action'],
-			'source' => EVENT_SOURCE_TRIGGERS,
-			'object' => EVENT_OBJECT_TRIGGER,
-			'eventid_till' => $eventid_till,
-			'objectids' => $trigger['triggerid'],
-			'value' => TRIGGER_VALUE_TRUE,
-			'sortfield' => ['eventid'],
-			'sortorder' => ZBX_SORT_DOWN,
-			'limit' => ZBX_WIDGET_ROWS
-		]);
-
-		CArrayHelper::sort($problems, [
-			['field' => 'clock', 'order' => ZBX_SORT_DOWN],
-			['field' => 'ns', 'order' => ZBX_SORT_DOWN]
-		]);
-
-		$r_eventids = [];
-
-		foreach ($problems as $problem) {
-			$r_eventids[$problem['r_eventid']] = true;
-		}
-		unset($r_eventids[0]);
-
-		$r_events = $r_eventids
-			? API::Event()->get([
-				'output' => ['clock', 'correlationid', 'userid'],
-				'source' => EVENT_SOURCE_TRIGGERS,
-				'object' => EVENT_OBJECT_TRIGGER,
-				'eventids' => array_keys($r_eventids),
-				'preservekeys' => true
-			])
-			: [];
-
-		$today = strtotime('today');
-		$last_clock = 0;
-
-		if ($problems && $show_tags != PROBLEMS_SHOW_TAGS_NONE) {
-			$tags = makeTags($problems, true, 'eventid', $show_tags, $filter_tags, $tag_name_format, $tag_priority);
-		}
-
-		$url_details = $allowed['ui_problems']
-			? (new CUrl('tr_events.php'))
-				->setArgument('triggerid', '')
-				->setArgument('eventid', '')
-			: null;
-
-		foreach ($problems as $problem) {
-			if (array_key_exists($problem['r_eventid'], $r_events)) {
-				$problem['r_clock'] = $r_events[$problem['r_eventid']]['clock'];
-				$problem['correlationid'] = $r_events[$problem['r_eventid']]['correlationid'];
-				$problem['userid'] = $r_events[$problem['r_eventid']]['userid'];
-			}
-			else {
-				$problem['r_clock'] = 0;
-				$problem['correlationid'] = 0;
-				$problem['userid'] = 0;
-			}
-
-			$can_be_closed = $allowed['close'];
-
-			if ($problem['r_eventid'] != 0) {
-				$value = TRIGGER_VALUE_FALSE;
-				$value_str = _('RESOLVED');
-				$value_clock = $problem['r_clock'];
-				$can_be_closed = false;
-			}
-			else {
-				$in_closing = false;
-
-				foreach ($problem['acknowledges'] as $acknowledge) {
-					if (($acknowledge['action'] & ZBX_PROBLEM_UPDATE_CLOSE) == ZBX_PROBLEM_UPDATE_CLOSE) {
-						$in_closing = true;
-						$can_be_closed = false;
-						break;
-					}
-				}
-
-				$value = $in_closing ? TRIGGER_VALUE_FALSE : TRIGGER_VALUE_TRUE;
-				$value_str = $in_closing ? _('CLOSING') : _('PROBLEM');
-				$value_clock = $in_closing ? time() : $problem['clock'];
-			}
-
-			$cell_clock = ($problem['clock'] >= $today)
-				? zbx_date2str(TIME_FORMAT_SECONDS, $problem['clock'])
-				: zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['clock']);
-
-			if ($url_details !== null) {
-				$url_details
-					->setArgument('triggerid', $problem['objectid'])
-					->setArgument('eventid', $problem['eventid']);
-				$cell_clock = new CCol(new CLink($cell_clock, $url_details));
-			}
-			else {
-				$cell_clock = new CCol($cell_clock);
-			}
-
-			if ($problem['r_eventid'] != 0) {
-				$cell_r_clock = ($problem['r_clock'] >= $today)
-					? zbx_date2str(TIME_FORMAT_SECONDS, $problem['r_clock'])
-					: zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['r_clock']);
-				$cell_r_clock = (new CCol(($url_details !== null)
-					? new CLink($cell_r_clock, $url_details)
-					: $cell_r_clock
-				))
-					->addClass(ZBX_STYLE_NOWRAP)
-					->addClass(ZBX_STYLE_RIGHT);
-			}
-			else {
-				$cell_r_clock = '';
-			}
-
-			$is_acknowledged = ($problem['acknowledged'] == EVENT_ACKNOWLEDGED);
-			$cell_status = new CSpan($value_str);
-
-			// add colors and blinking to span depending on configuration and trigger parameters
-			addTriggerValueStyle($cell_status, $value, $value_clock, $is_acknowledged);
-
-			if ($show_timeline) {
-				if ($last_clock != 0) {
-					CScreenProblem::addTimelineBreakpoint($table, $last_clock, $problem['clock'], ZBX_SORT_DOWN);
-				}
-				$last_clock = $problem['clock'];
-
-				$row = [
-					$cell_clock->addClass(ZBX_STYLE_TIMELINE_DATE),
-					(new CCol())
-						->addClass(ZBX_STYLE_TIMELINE_AXIS)
-						->addClass(ZBX_STYLE_TIMELINE_DOT),
-					(new CCol())->addClass(ZBX_STYLE_TIMELINE_TD)
-				];
-			}
-			else {
-				$row = [
-					$cell_clock
-						->addClass(ZBX_STYLE_NOWRAP)
-						->addClass(ZBX_STYLE_RIGHT)
-				];
-			}
-
-			// Create acknowledge link.
-			$problem_update_link = ($allowed['add_comments'] || $allowed['change_severity'] || $allowed['acknowledge']
-					|| $can_be_closed)
-				? (new CLink($is_acknowledged ? _('Yes') : _('No')))
-					->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-					->addClass(ZBX_STYLE_LINK_ALT)
-					->onClick('acknowledgePopUp('.json_encode(['eventids' => [$problem['eventid']]]).', this);')
-				: (new CSpan($is_acknowledged ? _('Yes') : _('No')))->addClass(
-					$is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED
-				);
-
-			$table->addRow(array_merge($row, [
-				$cell_r_clock,
-				$cell_status,
-				(new CCol(
-					($problem['r_eventid'] != 0)
-						? zbx_date2age($problem['clock'], $problem['r_clock'])
-						: zbx_date2age($problem['clock'])
-				))
-					->addClass(ZBX_STYLE_NOWRAP),
-				$problem_update_link,
-				($show_tags != PROBLEMS_SHOW_TAGS_NONE) ? $tags[$problem['eventid']] : null
-			]));
-		}
-	}
-
-	$div->addItem($table);
-
-	return $div;
-}
-
 /**
  * Place filter tags at the beginning of tags array.
  *
